SystemOrganization addCategory: #'STON-Core'!

!Dictionary class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 14:59'!
fromSton: stonReader
	"Instances of STON mapClass will be read directly and won't arrive here.
	Other (sub)classes will use this method."
	
	| dictionary |
	dictionary := self new.
	stonReader parseMapDo: [ :key :value |
		dictionary at: key put: value ].
	^ dictionary! !

!Dictionary methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 14:58'!
stonOn: stonWriter
	"Instances of STON mapClass will be encoded directly, without a class tag.
	Other (sub)classes will be encoded with a class tag and will use a map representation. "
	
	self class == STON mapClass
		ifTrue: [ 
			stonWriter writeMap: self ]
		ifFalse: [ 
			stonWriter 
				writeObject: self 
				do: [ stonWriter encodeMap: self ] ]! !

!Dictionary methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 4/14/2016 13:55'!
stonPostReferenceResolution
	"When references were resolved in me, the hash of my keys might have changed.
	Check if I am still healthy and rehash me if not."
	
	self isHealthy ifFalse: [ self rehash ]! !

!Set methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 4/14/2016 13:55'!
stonPostReferenceResolution
	"When references were resolved in me, the hash of my elements might have changed.
	Check if I am still healthy and rehash me if not."
	
	self isHealthy ifFalse: [ self rehash ]! !

!Object class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 19:58'!
fromSton: stonReader
	"Create a new instance and delegate decoding to instance side.
	Override only when new instance should be created directly (see implementors). "
	
	^ self new
		fromSton: stonReader;
		yourself! !

!Object class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/27/2014 11:08'!
stonAllInstVarNames
	"Override to encode my instances using a custom set of instance variables or to define their order."
	
	^ self allInstVarNames ! !

!Object class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 8/12/2014 13:28'!
stonName
	"Override to encode my instances using a different class name."
	
	^ self name! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 5/28/2015 15:32'!
fromSton: stonReader
	"Decode non-variable classes from a map of their instance variables and values.
	Override to customize and add a matching #toSton: (see implementors)."
	
	self class isVariable 
		ifTrue: [
			stonReader error: 'custom #fromSton: implementation needed for variable/indexable class' ]
		ifFalse: [
			stonReader parseNamedInstVarsFor: self ]! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/18/2012 20:16'!
isStonReference
	^ false! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:24'!
stonContainSubObjects
	"Return true if I contain subObjects that should be processed, false otherwise.
	Overwrite when necessary. See also #stonProcessSubObjects:"
	
	^ true! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 5/28/2015 15:06'!
stonOn: stonWriter
	"Encode non-variable classes with a map of their instance variable and values.
	Override to customize and add a matching #fromSton: (see implementors)."

	self class isVariable 
		ifTrue: [
			stonWriter error: 'custom #stonOn: implementation needed for variable/indexable class' ]
		ifFalse: [
			stonWriter writeObject: self ]
! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 4/14/2016 13:45'!
stonPostReferenceResolution
	"Hook that is called when references were resolved processing this object or one of its sub objects. This will most probably influence hash values. Override to take appropriate action."! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:22'!
stonProcessSubObjects: block
	"Execute block to (potentially) change each of my subObjects.
	In general, all instance and indexable variables are processed.
	Overwrite when necessary. Not used when #stonContainSubObjects returns false."
	
	1 to: self class instSize do: [ :each |
		self instVarAt: each put: (block value: (self instVarAt: each)) ].
	(self class isVariable and: [ self class isBytes not ])
		ifTrue: [
			1 to: self basicSize do: [ :each |
				self basicAt: each put: (block value: (self basicAt: each)) ] ]! !

!Object methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/13/2013 23:43'!
stonShouldWriteNilInstVars
	"Return true if my instance variables that are nil should be written out, 
	false otherwise. Overwrite when necessary. By default, return false."
	
	^ false! !

Object subclass: #STON
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STON commentStamp: 'SvenVanCaekenberghe 3/15/2016 14:23' prior: 0!
STON implements serialization and materialization using the Smalltalk Object Notation format.

I am a class side facade offering a high level API to write and read objects using STON.

U s a g e

Basic operations

  #toString:
  #fromString:

  STON toString: DisplayScreen boundingBox.
  STON fromString:  'Rectangle{#origin:Point[0,0],#corner:Point[1920,1030]}'.

  STON toString: { DateAndTime now. Float pi. 1 to: 10 by: 2. 3 days }.
  STON fromString:  '[DateAndTime[''2016-03-15T13:57:59.462422+01:00''],3.141592653589793,Interval{#start:1,#stop:10,#step:2},Duration{#nanos:0,#seconds:259200}]'

You can also read from or write to streams

  #fromStream:
  #put:onStream:

There is also the option to do pretty printing (indenting, multi line output) 

  #toStringPretty:
  #put:onStreamPretty:

STON is more or less a superset of JSON and is backwards compatible with JSON while parsing, and can be compatible with it while writing. The important differences (and the whole reason why STON exists in the first place) are 

  - class information (except for lists (Array) and maps (Dictionary))
  - proper handling of shared and circular references
  - more Smalltalk like syntax (Symbols with #, single qouted Strings, nil instead of null)
  - more defined special types (Date, Time, DataAndTime, ByteArray, Point)

Parsing JSON is done using #fromString: or #fromStream: with the results being composed of Arrays and Dictionaries.

Writing objects as JSON is done using: 

  #toJsonString[Pretty]:
  #put:asJsonOnStream[Pretty]:

Note that you can only write Arrays and Dictionaries !!

For a much more sophisticated JSON parser/writer implementation, have a look at NeoJSON.

Like JSON, STON does not allow for comments. However, a preprocessor option can skip C style comments before parsing.

I also define some contants used in the implementation: the class used as list, map and association, as well as the optional class name key (used when reading objects using an unknown class).


I m p l e m e n t a t i o n

The 2 key methods are

  #stonOn:
  #fromSton:

which work together with STONWriter and STONReader; read their class comments for all configuration options (you can use the #reader and #writer methods to avoid referring to these classes directly).

Several methods are used to support and/or control the implementation

  #stonName - defines the external name for a class
  #stonAllInstVarNames - defines which instance variable to write
  #stonContainSubObjects - shortcut looking into objects for subobjects
  #stonShouldWriteNilInstVars - option to skip writing nil valued instance variables


S y n t a x

	value
	  primitive-value
	  object-value
	  reference
	  nil
	primitive-value
	  number
	  true
	  false
	  symbol
	  string
	object-value
	  object
	  map
	  list
	object
	  classname map
	  classname list
	reference
	  @ int-index-previous-object-value
	map
	  {}
	  { members }
	members
	  pair
	  pair , members
	pair
	  string : value
	  symbol : value
	  number : value
	list
	  []
	  [ elements ]
	elements
	  value 
	  value , elements
	string
	  ''
	  ' chars '
	chars
	  char
	  char chars
	char
	  any-printable-ASCII-character-
	    except-'-"-or-\
	  \'
	  \"
	  \\
	  \/
	  \b
	  \f
	  \n
	  \r
	  \t
	  \u four-hex-digits
	symbol
	  # chars-limited
	  # ' chars '
	chars-limited
	  char-limited
	  char-limited chars-limited
	char-limited
	  a-z A-Z 0-9 - _ . /
	classname
	  uppercase-alpha-char alphanumeric-char
	number
	  int
	  int frac
	  int exp
	  int frac exp
	int
	  digit
	  digit1-9 digits 
	  - digit
	  - digit1-9 digits
	frac
	  . digits
	exp
	  e digits
	digits
	  digit
	  digit digits
	e
	  e
	  e+
	  e-
	  E
	  E+
	  E-
!

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/15/2015 17:11'!
associationClass
	^ Association! !

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/12/2014 15:04'!
classNameKey
	^ #className! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 1/24/2012 09:39'!
fromStream: readStream
	^ (self reader on: readStream) next! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 6/9/2015 10:03'!
fromStreamWithComments: readStream
	^ (self reader on: (STONCStyleCommentsSkipStream on: readStream)) next! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 1/24/2012 09:40'!
fromString: string
	^ self fromStream: string readStream! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 6/9/2015 10:03'!
fromStringWithComments: string
	^ self fromStreamWithComments: string readStream! !

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/15/2016 14:40'!
jsonWriter
	^ STONWriter new
			jsonMode: true;
			referencePolicy: #error;
			yourself! !

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/24/2012 09:36'!
listClass
	^ Array! !

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/24/2012 09:36'!
mapClass
	^ Dictionary! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 5/30/2012 22:06'!
put: object asJsonOnStream: stream
	(self jsonWriter on: stream) nextPut: object! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 5/30/2012 22:07'!
put: object asJsonOnStreamPretty: stream
	(self jsonWriter on: stream)
		prettyPrint: true; 
		nextPut: object! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 1/24/2012 09:49'!
put: object onStream: stream
	(self writer on: stream) nextPut: object! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 2/12/2012 19:30'!
put: object onStreamPretty: stream
	(self writer on: stream)
		prettyPrint: true; 
		nextPut: object! !

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/24/2012 09:37'!
reader
	^ STONReader new! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 5/30/2012 22:07'!
toJsonString: object
	^ String streamContents: [ :stream |
		self put: object asJsonOnStream: stream ]! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 5/30/2012 22:07'!
toJsonStringPretty: object
	^ String streamContents: [ :stream |
		self put: object asJsonOnStreamPretty: stream ]! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 1/24/2012 09:49'!
toString: object
	^ String streamContents: [ :stream |
		self put: object onStream: stream ]! !

!STON class methodsFor: 'convencience' stamp: 'SvenVanCaekenberghe 2/12/2012 19:30'!
toStringPretty: object
	^ String streamContents: [ :stream |
		self put: object onStreamPretty: stream ]! !

!STON class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/24/2012 09:37'!
writer
	^ STONWriter new! !

Object subclass: #STONCStyleCommentsSkipStream
	instanceVariableNames: 'stream peekedCharacter delimiter escape'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONCStyleCommentsSkipStream commentStamp: 'SvenVanCaekenberghe 7/17/2015 13:50' prior: 0!
I am STONCStyleCommentsSkipStream, a character ReadStream wrapping another character ReadStream. I skip C style comments, much like a classic C preprocessor.

C, C++, Java, JavaScript style comments are either the multiline

	/* a comment */
	
or the single line, up to end of line

	// a comment 
	
I deal with any end of line convention. Multiline comments cannot be nested. 

You create me #on: another character ReadStream. 

Here is an example:

	(STONCStyleCommentsSkipStream on: 'abc/*comment*/def' readStream) upToEnd.
	
Comments inside single and double quote delimited strings are ignored. Backslash escapes for single and double quotes inside strings are honored.
!

!STONCStyleCommentsSkipStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/8/2015 22:58'!
on: readStream
	^ self new
		on: readStream;
		yourself! !

!STONCStyleCommentsSkipStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2015 23:24'!
atEnd
	^ self peek isNil! !

!STONCStyleCommentsSkipStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/8/2015 22:58'!
close
	stream close! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:55'!
collectionSpecies
	^ String! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/17/2015 15:30'!
consumeComment
	stream peek = $/ ifTrue: [ self consumeToEOL ].
	stream peek = $* ifTrue: [ self consumeToCommentEnd ]! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/8/2015 23:21'!
consumeToCommentEnd
	[ stream atEnd or: [ stream next = $* and: [ stream peekFor: $/ ] ] ] whileFalse! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/8/2015 23:11'!
consumeToEOL
	| eol char |
	eol := false.
	[ eol ] whileFalse: [ 
		char := stream next.
		(char isNil or: [ char = Character lf ])
			ifTrue: [ eol := true ]
			ifFalse: [ 
				char = Character cr
					ifTrue: [ 
						eol := true.
						stream peekFor: Character lf ] ] ]! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/17/2015 14:21'!
escape
	"Return true when we previously read a backslash escape inside a string, 
	so that the next string delimiter should be returned as is"
	
	^ escape = true! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/17/2015 15:31'!
handleStringDelimiter: char
	self escape
		ifTrue: [ escape := false ]
		ifFalse: [ 
			self insideString
				ifTrue: [ 
					char = delimiter
						ifTrue: [ delimiter := nil ] ]
				ifFalse: [ delimiter := char ] ].
	^ char! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/17/2015 14:21'!
insideString
	"Return true when we are currently inside a string where comments should be ignored."
	
	^ (delimiter = $') | (delimiter = $")! !

!STONCStyleCommentsSkipStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:58'!
isBinary
	^ false! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 23:06'!
next
	^ peekedCharacter
		ifNil: [ 
			stream atEnd
				ifFalse: [ self nextNonCommentChar ] ]
		ifNotNil: [ | character |
			character := peekedCharacter.
			peekedCharacter := nil.
			character ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
nextLine
	"Read a CR, LF or CRLF terminated line, returning the contents of the line without the EOL. Return nil when the receiver is #atEnd."
	
	self atEnd ifTrue: [ ^ nil ].
	^ self collectionSpecies streamContents: [ :out | | eol char |
		eol := false.
		[ eol ] whileFalse: [ 
			char := self next.
			(char isNil or: [ char = Character lf ])
				ifTrue: [ eol := true ]
				ifFalse: [ 
					char = Character cr 
						ifTrue: [ eol := true. self peekFor: Character lf ]
						ifFalse: [  out nextPut: char ] ] ] ]! !

!STONCStyleCommentsSkipStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/17/2015 15:32'!
nextNonCommentChar
	| char |
	char := stream next.
	(self insideString and: [ char = $\ ])
		ifTrue: [ 
			escape := true. 
			^ char ].
	(char = $') | (char = $")
		ifTrue: [ 
			^ self handleStringDelimiter: char ].
	escape := false. 
	^ (char = $/ and: [ self insideString not ])
		ifTrue: [ 
			self consumeComment.
			stream next ]
		ifFalse: [ char ]
! !

!STONCStyleCommentsSkipStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/8/2015 22:58'!
on: readStream
	stream := readStream! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 23:06'!
peek
	^ peekedCharacter
		ifNil: [
			stream atEnd
				ifFalse: [ 
					peekedCharacter := self nextNonCommentChar ] ]
! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
peekFor: object
	^ self peek = object
		ifTrue: [ 
			self next.
			true ]
		ifFalse: [ false ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2015 10:01'!
position
	^ stream position! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
readInto: collection startingAt: offset count: requestedCount 
	"Read count elements and place them in collection starting at offset.
	Return the number of elements actually read."
	
	^ peekedCharacter 
		ifNil: [ | readCount |
			[ readCount := self encoder 
					readInto: collection 
					startingAt: offset 
					count: requestedCount 
					fromStream: stream ]
				on: ZnByteStringBecameWideString 
				do: [ :byteStringBecameWideString | 
					byteStringBecameWideString becomeForward; resume ].
			readCount ]
		ifNotNil: [ 
			collection at: offset put: peekedCharacter.
			peekedCharacter := nil.
			(self 
				readInto: collection 
				startingAt: offset + 1
				count: requestedCount - 1) + 1 ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
skip: count
	count timesRepeat: [ self next ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
upTo: anObject 	
	^ self collectionSpecies 
		streamContents: [ :out | | element |
			[ self atEnd or: [ (element := self next) = anObject ] ] whileFalse: [ 
				out nextPut: element ] ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
upToEnd
	^ self collectionSpecies
		streamContents: [ :collectionStream | 
			[ self atEnd ] whileFalse: [ collectionStream nextPut: self next ] ]! !

!STONCStyleCommentsSkipStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/8/2015 22:56'!
wrappedStream
	^ stream! !

Object subclass: #STONJSON
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONJSON commentStamp: 'SvenVanCaekenberghe 3/15/2016 15:16' prior: 0!
I am STONJSON, a class side facade to use STON as a simple JSON parser/writer.

STON is more or less a superset of JSON and is backwards compatible with JSON while parsing, and can be compatible with it while writing. The important differences (and the whole reason why STON exists in the first place) are 

  - class information (except for lists (Array) and maps (Dictionary))
  - proper handling of shared and circular references
  - more Smalltalk like syntax (Symbols with #, single qouted Strings, nil instead of null)
  - more defined special types (Date, Time, DataAndTime, ByteArray, Point)

Parsing JSON is done using

  #fromString:
  #fromStream: 

with the results being composed of Arrays and Dictionaries.

Writing objects as JSON is done using

  #toString[Pretty]:
  #put:onStream[Pretty]:

Note that you can only write Arrays and Dictionaries !! Shared and circular references will be noted and signalled using an exception.

E x a m p l e s

  STONJSON toString: { 1. -1. Float pi. true. 'JSON' }.
  STONJSON fromString: '[1,-1,3.141592653589793,true,"JSON"]'.

  STONJSON toStringPretty: { #foo->1. #bar->2 } asDictionary.
  STONJSON fromString: '{"foo":1,"bar":2,"sub":{"a":true,"b":false},"flags":[1,8,32]}'.
 
For a much more sophisticated JSON parser/writer implementation, have a look at NeoJSON.!

!STONJSON class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/15/2016 14:28'!
fromStream: stream
	^ STON fromStream: stream! !

!STONJSON class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/15/2016 14:28'!
fromString: string
	^ STON fromString: string! !

!STONJSON class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/15/2016 14:30'!
put: object onStream: stream
	STON put: object asJsonOnStream: stream! !

!STONJSON class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/15/2016 14:30'!
put: object onStreamPretty: stream
	STON put: object asJsonOnStreamPretty: stream! !

!STONJSON class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/15/2016 14:30'!
toString: object
	^ STON toJsonString: object! !

!STONJSON class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/15/2016 14:30'!
toStringPretty: object
	^ STON toJsonStringPretty: object! !

Object subclass: #STONReader
	instanceVariableNames: 'readStream objects classes unresolvedReferences stringStream allowComplexMapKeys acceptUnknownClasses newLine convertNewLines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONReader commentStamp: 'SvenVanCaekenberghe 4/26/2015 23:49' prior: 0!
STONReader materializes objects using the Smalltalk Object Notation format.

This parser is backwards compatible with standard JSON.

Customization options are:

- allowComplexMapKeys <Boolean> default is false
	if true, any object is allowed as map key
	if false, only strings, symbols and numbers are allowed as map keys
- acceptUnknownClasses <Boolean> default is false
	if true, unknown class names are allowed, the standard #mapClass (Dictionary) is instanciated and the class name is added under #classNameKey (#className)
	if false, unknown class names result in a NotFound error
- convertNewLines <Boolean> default is false
	if true, any unescaped EOL sequence CR, LF or CRLF inside strings or symbols is read and converted as the chosen EOL sequence
	if false, CR, LF and CRLF are read unmodified
- newLine <String> default is String cr
	sequence to use as EOL!

!STONReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/14/2012 20:25'!
on: readStream
	^ self new
		on: readStream;
		yourself! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/12/2014 14:39'!
acceptUnknownClasses: boolean
	acceptUnknownClasses := boolean! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/25/2012 15:24'!
allowComplexMapKeys: boolean
	allowComplexMapKeys := boolean! !

!STONReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/7/2012 13:49'!
atEnd
	^ readStream atEnd! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/14/2012 20:26'!
close
	readStream ifNotNil: [
		readStream close.
		readStream := nil ]! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/6/2012 20:59'!
consumeWhitespace
	"Strip whitespaces from the input stream."

	[ readStream atEnd not and: [ readStream peek isSeparator ] ]
		whileTrue: [ readStream next ]! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/26/2015 23:39'!
convertNewLines: boolean
	"When true, any newline CR, LF or CRLF read unescaped inside strings or symbols 
	will be converted to the newline convention chosen, see #newLine:
	The default is false, not doing any convertions."
	
	convertNewLines := boolean! !

!STONReader methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/13/2013 16:13'!
error: aString
	| streamPosition |
	"Remain compatible with streams that don't understand #position"
	streamPosition := [ readStream position ]
		on: MessageNotUnderstood do: [ nil ].
	^ STONReaderError signal: aString streamPosition: streamPosition! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/7/2012 10:33'!
expectChar: character
	"Expect character and consume input and optional whitespace at the end,
	 throw an error otherwise."

	(self matchChar: character)
		ifFalse: [ self error: character asString, ' expected' ]! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/14/2016 13:57'!
initialize
	super initialize.
	objects := IdentityDictionary new.
	classes := IdentityDictionary new.
	allowComplexMapKeys := acceptUnknownClasses := convertNewLines := false.
	newLine := String cr.
	unresolvedReferences := 0! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/12/2012 19:27'!
isClassChar: char
	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789' includes: char! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/12/2012 19:27'!
isClassStartChar: char
	^ 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' includes: char! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/12/2012 19:28'!
isSimpleSymbolChar: char
	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_./' includes: char! !

!STONReader methodsFor: 'testing' stamp: 'ChristopheDemarey 11/7/2014 09:36'!
isValidPropertyNameType: associationKey
	^ allowComplexMapKeys or: [associationKey isString or: [ associationKey isNumber ] ]! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/12/2014 14:41'!
lookupClass: name
	Smalltalk globals 
		at: name 
		ifPresent: [ :class | ^ class ].
	^ classes 
		at: name 
		ifAbsentPut: [
			Object allSubclasses 
				detect: [ :class | class isMeta not and: [ class stonName == name ]  ]
				ifNone: [ NotFound signalFor: name ] ]
	! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2012 10:57'!
match: string do: block
	"Try to read and consume string and execute block if successful.
	Else do nothing (but do not back up)"

	(string allSatisfy: [ :each | readStream peekFor: each ])
		ifTrue: [ 
			self consumeWhitespace.
			block value ]! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/3/2012 10:54'!
matchChar: character
	"Tries to match character, consume input and 
	answer true if successful and consumes whitespace at the end."

	^ (readStream peekFor: character)
		ifTrue: [ 
			self consumeWhitespace.
			true ]
		ifFalse: [ false ]! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/26/2015 22:48'!
newLine: string
	"Set the newline convention to be used when converting newlines, see #convertNewLines"
	
	newLine := string ! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/18/2012 20:54'!
newReference
	| index reference |
	index := objects size + 1.
	reference := STONReference index: index.
	objects at: index put: reference.
	^ reference! !

!STONReader methodsFor: 'public' stamp: 'SvenVanCaekenberghe 4/14/2016 13:57'!
next
	| object |
	self consumeWhitespace.
	object := self parseValue.
	unresolvedReferences > 0
		ifTrue: [ self processSubObjectsOf: object ].
	unresolvedReferences = 0
		ifFalse: [ self error: 'Inconsistent reference resolution' ].
	^ object! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/14/2012 20:26'!
on: aReadStream
	readStream := aReadStream! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/14/2016 13:57'!
optimizeForLargeStructures
	self class environment
		at: #FLLargeIdentityDictionary
		ifPresent: [ :identityDictionaryClass | objects := identityDictionaryClass new ]! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 4/26/2015 22:40'!
parseCharacter
	| char |
	^ (char := readStream next) = $\
		ifTrue: [ self parseEscapedCharacter ]
		ifFalse: [ char ]! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 4/26/2015 23:02'!
parseCharacterConvertingNewLinesOn: writeStream
	| char |
	(char := readStream next) = $\
		ifTrue: [ writeStream nextPut: self parseEscapedCharacter ]
		ifFalse: [ 
			char = Character lf 
				ifTrue: [ writeStream nextPutAll: newLine ]
				ifFalse: [ 
					char = Character cr 
						ifTrue: [ 
							readStream peekFor: Character lf.
							writeStream nextPutAll: newLine ]
						ifFalse: [ writeStream nextPut: char ] ] ]! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2016 20:06'!
parseCharacterHex
	| value codePoint |
	value := self parseCharacterHex4Value.
	(value < 16rD800 or: [ value > 16rDBFF ])
		ifTrue: [ codePoint := value ]
		ifFalse: [ | leadSurrogate trailSurrogate |
			"Characters not in the Basic Multilingual Plane are encoded as a UTF-16 surrogate pair"
			"See https://tools.ietf.org/html/rfc7159#section-7"
			leadSurrogate := value.
			trailSurrogate := self parseTrailingSurrogateHexEscape.
			codePoint := (leadSurrogate - 16rD800) * 16r400 + (trailSurrogate - 16rDC00).
			codePoint := 16r10000 + codePoint ].
	^ Character codePoint: codePoint! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2016 20:06'!
parseCharacterHex4Value
	| value |
	value := self parseCharacterHexDigit.
	3 timesRepeat: [ 
		value := (value << 4) + self parseCharacterHexDigit ].
	^ value! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 1/15/2012 11:28'!
parseCharacterHexDigit
	| digit |
	readStream atEnd ifFalse: [ 
		digit := readStream next asInteger.
		(digit between: "$0" 48 and: "$9" 57)
			ifTrue: [ ^ digit - 48 ].
		(digit between: "$A" 65 and: "$F" 70)
			ifTrue: [ ^ digit - 55 ].
		(digit between: "$a" 97 and: "$f" 102)
			ifTrue: [ ^ digit - 87 ] ].
	self error: 'hex-digit expected'! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 5/29/2012 21:13'!
parseClass
	| className |
	className := self stringStreamContents: [ :stream |
		[ readStream atEnd not and: [ self isClassChar: readStream peek ] ] whileTrue: [ 
			stream nextPut: readStream next ] ].
	self consumeWhitespace.
	^ self lookupClass: className asSymbol
	! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 5/7/2012 13:16'!
parseConstantDo: block
	"Parse and consume either true|false|nil|null and execute block 
	or else do nothing (but do not back up).
	Hand written implementation to avoid the use of #position:"
	
	(readStream peek = $t)
		ifTrue: [
			^ self match: 'true' do: [ block value: true ] ].
	(readStream peek = $f)
		ifTrue: [
			^ self match: 'false' do: [ block value: false ] ].
	(readStream peek = $n)
		ifTrue: [
			readStream next.
			(readStream peek = $i)
				ifTrue: [
					self match: 'il' do: [ block value: nil ] ].
			(readStream peek = $u)
				ifTrue: [
					self match: 'ull' do: [ block value: nil ] ] ]		
! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 4/26/2015 22:14'!
parseEscapedCharacter
	| char |
	char := readStream next.
	(#($' $" $/ $\) includes: char)
		ifTrue: [ ^ char ].
	char = $b
		ifTrue: [ ^ Character backspace ].
	char = $f
		ifTrue: [ ^ Character newPage ].
	char = $n
		ifTrue: [ ^ Character lf ].
	char = $r
		ifTrue: [ ^ Character cr ].
	char = $t
		ifTrue: [ ^ Character tab ].
	char = $u
		ifTrue: [ ^ self parseCharacterHex ].
	self error: 'invalid escape character \' , (String with: char).
	^ char! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/12/2012 15:21'!
parseList
	| reference array |
	reference := self newReference.
	array := STON listClass streamContents: [ :stream |
		self parseListDo: [ :each | stream nextPut: each ] ].
	self setReference: reference to: array.
	^ array! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 11/27/2014 10:58'!
parseListDo: block
	| index |
	self expectChar: $[.
	(self matchChar: $]) ifTrue: [ ^ self ]. "short cut for empty lists"
	index := 1.
	[ readStream atEnd ] whileFalse: [
		block cull: self parseValue cull: index.
		(self matchChar: $]) ifTrue: [ ^ self ].
		index := index + 1.
		self expectChar: $, ].
	self error: 'end of list expected'! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/12/2012 15:55'!
parseListSingleton
	| value |
	value := nil.
	self parseListDo: [ :each :index |
		index = 1 ifTrue: [ value := each ] ].
	^ value! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/12/2012 15:22'!
parseMap
	| map |
	map := STON mapClass new.
	self storeReference: map.
	self parseMapDo: [ :key :value |
		map at: key put: value ].
	^ map! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 11/27/2014 12:54'!
parseMapDo: block
	self expectChar: ${.
	(self matchChar: $}) ifTrue: [ ^ self ]. "short cut for empty maps"
	[ readStream atEnd ] whileFalse: [ | name value |
		name := self parseSimpleValue.
		(self isValidPropertyNameType: name)
			ifFalse: [ self error: 'unexpected property name type' ].
		self expectChar: $:.
		value := self parseValue.
		block value: name value: value.
		"The above is a more efficient way to say 'self parseValue' and using the returned association"
		(self matchChar: $}) ifTrue: [ ^ self ].
		self expectChar: $, ].
	self error: 'end of map expected'! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 5/28/2015 15:31'!
parseNamedInstVarsFor: anObject
	self parseMapDo: [ :instVarName :value |
		anObject instVarNamed: instVarName put: value ]! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 12/3/2012 11:08'!
parseNumber
	| negated number |
	negated := readStream peekFor: $-.
	number := self parseNumberInteger.
	(readStream peekFor: $.)
		ifTrue: [ number := number + self parseNumberFraction ].
	((readStream peekFor: $e) or: [ readStream peekFor: $E ])
		ifTrue: [ number := number * self parseNumberExponent ].
	negated
		ifTrue: [ number := number negated ].
	self consumeWhitespace.
	^ number! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 12/3/2012 11:08'!
parseNumberExponent
	| number negated |
	number := 0.
	(negated := readStream peekFor: $-)
		ifFalse: [ readStream peekFor: $+ ].
	[ readStream atEnd not and: [ readStream peek isDigit ] ]
		whileTrue: [ number := 10 * number + readStream next digitValue ].
	negated
		ifTrue: [ number := number negated ].
	^ 10 raisedTo: number! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 1/15/2012 10:52'!
parseNumberFraction
	| number power |
	number := 0.
	power := 1.0.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] whileTrue: [
		number := 10 * number + readStream next digitValue.
		power := power * 10.0 ].
	^ number / power! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 5/29/2012 21:13'!
parseNumberInteger
	| number |
	number := 0.
	[ readStream atEnd not and: [ readStream peek isDigit ] ] whileTrue: [ 
		number := 10 * number + readStream next digitValue ].
	^ number! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/12/2014 15:04'!
parseObject
	| targetClass reference object |
	[
		reference := self newReference.
		targetClass := self parseClass.
		object := targetClass fromSton: self .
		self setReference: reference to: object ]
		on: NotFound 
		do: [ :notFound |
			acceptUnknownClasses 
				ifTrue: [ 
					object := STON mapClass new.
					self storeReference: object.
					self parseMapDo: [ :key :value |
						object at: key put: value ].
					object at: STON classNameKey put: notFound object ]
				ifFalse: [ self error: 'Cannot resolve class named ', notFound object printString ] ].
	^ object! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 5/28/2015 14:13'!
parseReference
	| index |
	self expectChar: $@.
	index := self parseNumberInteger.
	self consumeWhitespace.
	unresolvedReferences := unresolvedReferences + 1.
	^ STONReference index: index! !

!STONReader methodsFor: 'parsing' stamp: 'ChristopheDemarey 11/6/2014 15:28'!
parseSimpleValue
	| char |
	readStream atEnd ifFalse: [ 
		(self isClassStartChar: (char := readStream peek)) 
			ifTrue: [ ^ self parseObject ].
		char = ${
			ifTrue: [ ^ self parseMap ].
		char = $[
			ifTrue: [ ^ self parseList ].
		(char = $' or: [ char = $" ])
			ifTrue: [ ^ self parseString ].
		char = $#
			ifTrue: [ ^ self parseSymbol ].
		char = $@
			ifTrue: [ ^ self parseReference ].
		(char = $- or: [ char isDigit ])
			ifTrue: [ ^ self parseNumber ].
		self parseConstantDo: [ :value | ^ value ] ].
	self error: 'invalid input'! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 2/29/2012 21:50'!
parseString
	^ self parseStringInternal! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 4/26/2015 23:39'!
parseStringInternal
	| result delimiter |
	delimiter := readStream next.
	(delimiter = $' or: [ delimiter = $" ])
		ifFalse: [ self error: ''' or " expected' ].
	result := self
		stringStreamContents: [ :stream | 
			convertNewLines
				ifTrue: [ 
					[ readStream atEnd or: [ readStream peek = delimiter ] ] 
						whileFalse: [ self parseCharacterConvertingNewLinesOn: stream ] ]
				ifFalse: [ 
					[ readStream atEnd or: [ readStream peek = delimiter ] ] 
						whileFalse: [ stream nextPut: self parseCharacter ] ] ].
	self expectChar: delimiter.
	^ result! !

!STONReader methodsFor: 'parsing-internal' stamp: 'SvenVanCaekenberghe 5/7/2012 10:42'!
parseSymbol
	| string |
	self expectChar: $#.
	readStream peek = $'
		ifTrue: [ ^ self parseStringInternal asSymbol ].
	string := self stringStreamContents: [ :stream |
		[ readStream atEnd not and: [ self isSimpleSymbolChar: readStream peek ] ] whileTrue: [
			stream nextPut: readStream next ] ].
	string isEmpty
		ifFalse: [ 
			self consumeWhitespace.
			^ string asSymbol ].
	self error: 'unexpected input'! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2016 20:06'!
parseTrailingSurrogateHexEscape
	(readStream next = $\ and: [ readStream next = $u ])
		ifTrue: [ ^ self parseCharacterHex4Value ]
		ifFalse: [ self error: 'trailing surrogate hex escape expected' ]! !

!STONReader methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/15/2015 17:12'!
parseValue
	| value |
	value := self parseSimpleValue.
	^ (self matchChar: $:)
		ifTrue: [ 
			(self isValidPropertyNameType: value)
				ifFalse: [ self error: 'unexpected property name type' ].
			STON associationClass key: value value: self parseValue ]
		ifFalse: [ value ]! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/14/2016 13:43'!
processSubObjectsOf: object
	| unresolvedReferencesCount |
	unresolvedReferencesCount := unresolvedReferences.
	object stonProcessSubObjects: [ :each | 
		each isStonReference
			ifTrue: [ self resolveReference: each ]
			ifFalse: [ 
				each stonContainSubObjects
					ifTrue: [ self processSubObjectsOf: each ]
					ifFalse: [ each ] ] ].
	unresolvedReferencesCount > unresolvedReferences
		ifTrue: [ object stonPostReferenceResolution ].
	^ object! !

!STONReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/25/2012 17:37'!
reset
	unresolvedReferences := 0.
	objects removeAll! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/14/2016 13:07'!
resolveReference: reference
	unresolvedReferences := unresolvedReferences - 1.
	^ self resolveReferenceIndex: reference index! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/18/2012 21:03'!
resolveReferenceIndex: index
	^ objects at: index! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/18/2012 21:01'!
setReference: reference to: object
	objects at: reference index put: object! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/16/2012 11:10'!
storeReference: object
	| index |
	index := objects size + 1.
	objects at: index put: object.
	^ index! !

!STONReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/9/2012 17:23'!
stringStreamContents: block
	stringStream ifNil: [ 
		stringStream := (String new: 32) writeStream ].
	stringStream reset.
	block value: stringStream.
	^ stringStream contents! !

Object subclass: #STONReference
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONReference commentStamp: 'SvenVanCaekenberghe 1/22/2012 20:13' prior: 0!
STONReference holds a forward reference to another object during materialization.
!

!STONReference class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/18/2012 19:37'!
index: integer
	^ self new
		index: integer;
		yourself! !

!STONReference methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/18/2012 20:23'!
= anObject
	^ self class == anObject class and: [ self index = anObject index ]! !

!STONReference methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/18/2012 20:22'!
hash
	^ index hash! !

!STONReference methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/18/2012 19:35'!
index
	^ index! !

!STONReference methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/18/2012 19:35'!
index: integer
	index := integer! !

!STONReference methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/18/2012 20:22'!
isStonReference
	^ true! !

!STONReference methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/28/2012 21:06'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: index; nextPut: $)! !

Object subclass: #STONStreamWriter
	instanceVariableNames: 'writer first'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONStreamWriter commentStamp: '<historical>' prior: 0!
STONStreamWriter helps in streaming writing STON representations.
This is an abstract class.!

STONStreamWriter subclass: #STONListWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONListWriter commentStamp: '<historical>' prior: 0!
STONArrayWriter helps in writing array based STON representations.
!

!STONListWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:03'!
add: anObject
	first ifTrue: [ first := false ] ifFalse: [ writer listElementSeparator ].
	writer nextPut: anObject! !

STONListWriter subclass: #STONShortListWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONShortListWriter commentStamp: '<historical>' prior: 0!
STONShortArrayWriter helps in writing short array based STON representations.
!

!STONShortListWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/12/2012 15:49'!
add: anObject
	first ifTrue: [ first := false ] ifFalse: [ writer shortListElementSeparator ].
	writer nextPut: anObject! !

STONStreamWriter subclass: #STONMapWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONMapWriter commentStamp: '<historical>' prior: 0!
STONDictionaryWriter helps in writing dictionary based STON representations.!

!STONMapWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:03'!
at: key put: value
	first ifTrue: [ first := false ] ifFalse: [ writer mapElementSeparator ].
	writer encodeKey: key value: value! !

!STONStreamWriter class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/24/2012 09:29'!
on: stonWriter
	^ self new
		on: stonWriter;
		yourself! !

!STONStreamWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/24/2012 09:30'!
initialize
	super initialize.
	first := true! !

!STONStreamWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/24/2012 09:29'!
on: stonWriter
	writer := stonWriter! !

Object subclass: #STONWriter
	instanceVariableNames: 'writeStream prettyPrint newLine asciiOnly jsonMode keepNewLines referencePolicy level objects'
	classVariableNames: 'STONCharacters STONSimpleSymbolCharacters'
	poolDictionaries: ''
	category: 'STON-Core'!

!STONWriter commentStamp: 'SvenVanCaekenberghe 7/2/2015 15:28' prior: 0!
STONWriter serializes objects using the Smalltalk Object Notation format. 

Customization options are:

- prettyPrint <Boolean> default is false
	if true, produce pretty printed output
- newLine <String> default is String cr
	what sequence to use for EOL
- asciiOnly <Boolean> default is false
   if true, use \u escapes for all non-ASCII characters
   most common control characters are still escaped
- jsonMode <Boolean> default is false
	if true, the follow changes occur
	- strings are delimited with double quotes
	- nil is encoded as null
	- symbols are treated as strings
	- only STON listClass and STON mapClass instances are allowed as composite objects
	it is wise to also use either #error or #ignore as referencePolicy to avoid references
- referencePolicy <#normal|#ignore|#error> default is #normal
	if #normal, track and count object references and use references to implement sharing and break cycles
	if #error, track object references and signal STONWriterError when a shared reference is encountered
	if #ignore, don't track object references which might loop forever on cycles
 - keepNewLines <Boolean> default is false
	if true, any newline sequence CR, LF or CRLF inside strings or symbols will not be escaped 
	but will be written as the newline EOF convention!

!STONWriter class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 2/9/2012 16:43'!
initialize
	self initializeSTONCharacters.
	self initializeSTONSimpleSymbolCharacters! !

STONWriter initialize!

!STONWriter class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 7/2/2015 15:09'!
initializeSTONCharacters
	| escapes |
	STONCharacters := Array new: 127.
	32 to: 126 do: [ :each | 
		STONCharacters at: each + 1 put: #pass ].
	escapes := #( 8 '\b' 9 '\t' 10 '\n' 12 '\f' 13 '\r' 34 '\"' 39 '\''' 92 '\\' ).
	escapes pairsDo: [ :code :escape | 
		STONCharacters at: code + 1 put: escape ]
	! !

!STONWriter class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 2/14/2012 17:50'!
initializeSTONSimpleSymbolCharacters
	"STONSimpleSymbolCharacters asArray collectWithIndex: [ :each :index |
		each isZero ifTrue: [ (index - 1) asCharacter ] ]."
		
	STONSimpleSymbolCharacters := ByteArray new: 256 withAll: 1.
	1 to: 256 do: [ :each | | char |
		char := (each - 1) asCharacter.
		(self isSimpleSymbolChar: char)
			ifTrue: [ 
				STONSimpleSymbolCharacters at: each put: 0 ] ]! !

!STONWriter class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/14/2012 17:49'!
isSimpleSymbolChar: char
	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_./' includes: char! !

!STONWriter class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/14/2012 15:48'!
on: writeStream
	^ self new
		on: writeStream;
		yourself! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/2/2015 15:11'!
asciiOnly: boolean
	asciiOnly := boolean! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/14/2012 15:37'!
close
	writeStream ifNotNil: [
		writeStream close.
		writeStream := nil ]! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2016 21:09'!
encodeCharacter: char
	| code encoding |
	"STONCharacters contains for the lower 127 characters (codes 0 to 126) either nil (unknown), 
	#pass (output as is, clean ASCII characters) or a full escape string"
	((code := char codePoint) < 127 and: [ (encoding := STONCharacters at: code + 1) notNil ])
		ifTrue: [ 
			encoding = #pass
				ifTrue: [ writeStream nextPut: char ]
				ifFalse: [ writeStream nextPutAll: encoding ] ]
		ifFalse: [
			"always escape Latin1 C1 controls, or when asciiOnly is true" 
			(code > 16r9F and: [ asciiOnly not ])
				ifTrue: [ writeStream nextPut: char ]
				ifFalse: [ self escapeUnicode: code ] ]
! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/23/2012 11:32'!
encodeKey: key value: value
	self nextPut: key.
	self prettyPrintSpace.
	writeStream nextPut: $:.
	self prettyPrintSpace.
	self nextPut: value! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/5/2012 16:56'!
encodeList: elements
	writeStream nextPut: $[.
	elements isEmpty
		ifTrue: [
			self prettyPrintSpace ]
		ifFalse: [
			self indentedDo: [
				self newlineIndent.
				elements 
					do: [ :each | self nextPut: each ]
					separatedBy: [ self listElementSeparator ] ].
			self newlineIndent ].
	writeStream nextPut: $]
! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/5/2012 16:56'!
encodeMap: pairs
	| first |
	first := true.
	writeStream nextPut: ${.
	pairs isEmpty
		ifTrue: [
			self prettyPrintSpace ]
		ifFalse: [
			self indentedDo: [
				self newlineIndent.
				pairs keysAndValuesDo: [ :key :value |
					first 
						ifTrue: [ first := false ] 
						ifFalse: [ self mapElementSeparator ].
					self encodeKey: key value: value ] ].
			self newlineIndent ].
	writeStream nextPut: $}! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2015 23:17'!
encodeString: string
	writeStream nextPut: (jsonMode ifTrue: [ $" ] ifFalse: [ $' ]).
	keepNewLines
		ifTrue: [ 
			self encodeStringKeepingNewLines: string ]
		ifFalse: [ 
			string do: [ :each | self encodeCharacter: each ] ].
	writeStream nextPut: (jsonMode ifTrue: [ $" ] ifFalse: [ $' ])! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/26/2015 23:16'!
encodeStringKeepingNewLines: string
	| input char |
	input := string readStream.
	[ input atEnd ]
		whileFalse: [ 
			char := input next.
			char = Character lf
				ifTrue: [ writeStream nextPutAll: newLine ]
				ifFalse: [ 
					char = Character cr
						ifTrue: [ 
							input peekFor: Character lf.
							writeStream nextPutAll: newLine ]
						ifFalse: [ self encodeCharacter: char ] ] ]! !

!STONWriter methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 5/28/2015 15:02'!
error: aString
	^ STONWriterError signal: aString! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2016 21:06'!
escapeUnicode4: codePoint
	writeStream nextPutAll: '\u'.
	codePoint printOn: writeStream base: 16 nDigits: 4! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/24/2016 21:06'!
escapeUnicode: codePoint
	codePoint <= 16rFFF
		ifTrue: [ self escapeUnicode4: codePoint ]
		ifFalse: [ 
			codePoint <= 16r10FFFF
				ifTrue: [ | leadSurrogate trailSurrogate shifted |
					"Characters not in the Basic Multilingual Plane are encoded as a UTF-16 surrogate pair"
					"See https://tools.ietf.org/html/rfc7159#section-7"
					shifted := codePoint - 16r10000.
					leadSurrogate := 16rD800 + (shifted // 16r400).
					trailSurrogate := 16rDC00 + (shifted \\ 16r400).
					self escapeUnicode4: leadSurrogate.
					self escapeUnicode4: trailSurrogate ]
				ifFalse: [ self error: 'Character Unicode code point outside encoder range' ] ]! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/23/2012 11:12'!
indentedDo: block
	level := level + 1.
	block value.
	level := level - 1! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/2/2015 15:11'!
initialize
	super initialize.
	prettyPrint := false.
	newLine := String cr.
	level := 0.
	referencePolicy := #normal.
	jsonMode := keepNewLines := asciiOnly := false.
	objects := IdentityDictionary new! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/15/2014 13:35'!
isSimpleSymbol: symbol
	symbol isEmpty ifTrue: [ ^ false ].
	^ (symbol class 
		findFirstInString: symbol 
		inSet: STONSimpleSymbolCharacters 
		startingAt: 1) = 0
! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/27/2012 13:53'!
jsonMode: boolean
	jsonMode := boolean! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/26/2015 23:30'!
keepNewLines: boolean
	"If true, any newline CR, LF or CRLF inside strings or symbols will not be escaped
	but will instead be converted to the newline convention chosen, see #newLine:
	The default is false, where CR, LF or CRLF will be enscaped unchanged."
	
	keepNewLines := boolean! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/29/2012 12:23:33'!
listElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/29/2012 12:23:33'!
mapElementSeparator
	writeStream nextPut: $,.
	self newlineIndent
! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/26/2015 23:34'!
newLine: string
	"The sequence to use when ending a line, either CR, LF or CRLF"
	
	newLine := string! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/7/2013 10:39'!
newlineIndent
	prettyPrint ifTrue: [ 
		writeStream nextPutAll: newLine.
		level timesRepeat: [ writeStream tab ] ]! !

!STONWriter methodsFor: 'public' stamp: 'SvenVanCaekenberghe 1/14/2012 15:47'!
nextPut: anObject
	anObject stonOn: self! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/29/2012 12:23:33'!
on: aWriteStream
	writeStream := aWriteStream
! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/29/2012 15:54'!
optimizeForLargeStructures
	self class environment
		at: #FLLargeIdentityDictionary
		ifPresent: [ :identityDictionaryClass | objects := identityDictionaryClass new ].! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/14/2012 15:36'!
prettyPrint: boolean
	prettyPrint := boolean! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/23/2012 11:18'!
prettyPrintSpace
	prettyPrint ifTrue: [ writeStream space ]
! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/29/2012 12:23:33'!
referencePolicy: policy
	self assert: ( #(#normal #ignore #error) includes: policy ).
	referencePolicy := policy
! !

!STONWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/14/2012 15:37'!
reset
	objects removeAll! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/29/2012 12:23:33'!
shortListElementSeparator
	writeStream nextPut: $,.
	self prettyPrintSpace
! !

!STONWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/28/2015 15:04'!
with: object do: block
	| index |
	referencePolicy = #ignore 
		ifTrue: [ ^ block value ].
	(index := objects at: object ifAbsent: [ nil ]) notNil
		ifTrue: [
			referencePolicy = #error
				ifTrue: [ ^ self error: 'shared reference detected' ].
			self writeReference: index ]
		ifFalse: [
			index := objects size + 1.
			objects at: object put: index.
			block value ]! !

!STONWriter methodsFor: 'writing' stamp: 'ChristopheDemarey 11/6/2014 15:07'!
writeAssociation: association
	self 
		encodeKey: association key
		value: association value! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/14/2012 15:51'!
writeBoolean: boolean
	writeStream print: boolean! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/14/2012 16:05'!
writeFloat: float
	writeStream print: float! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/14/2012 15:48'!
writeInteger: integer
	writeStream print: integer! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:00'!
writeList: collection
	self with: collection do: [ 
		self encodeList: collection ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:01'!
writeMap: hashedCollection
	self with: hashedCollection do: [ 
		self encodeMap: hashedCollection ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 4/27/2012 13:50'!
writeNull
	jsonMode
		ifTrue: [ writeStream nextPutAll: 'null' ]
		ifFalse: [ writeStream print: nil ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 11/27/2014 11:08'!
writeObject: anObject
	| instanceVariableNames |
	(instanceVariableNames := anObject class stonAllInstVarNames) isEmpty
		ifTrue: [ 
			self writeObject: anObject do: [ self encodeMap: #() ] ]
		ifFalse: [ 
			self writeObject: anObject streamMap: [ :dictionary | 
				instanceVariableNames do: [ :each | 
					(anObject instVarNamed: each)
						ifNotNil: [ :value | 
							dictionary at: each asSymbol put: value ]
						ifNil: [ 
							anObject stonShouldWriteNilInstVars 
								ifTrue: [ dictionary at: each asSymbol put: nil ] ] ] ] ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/28/2015 15:04'!
writeObject: anObject do: block
	(jsonMode and: [ anObject class ~= STON listClass and: [ anObject class ~= STON mapClass ] ])
		ifTrue: [ self error: 'wrong object class for JSON mode' ].
	self with: anObject do: [
		writeStream nextPutAll: anObject class stonName.
		self prettyPrintSpace.
		block value ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 2/12/2012 20:02'!
writeObject: object listSingleton: element
	self writeObject: object do: [
		writeStream nextPut: $[.
		self 
			prettyPrintSpace;
			nextPut: element;
			prettyPrintSpace.
		writeStream nextPut: $] ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:02'!
writeObject: object streamList: block
	self writeObject: object do: [ | listWriter |
		listWriter := STONListWriter on: self.
		writeStream nextPut: $[.
		self indentedDo: [
			self newlineIndent.
			block value: listWriter ].
		self newlineIndent.
		writeStream nextPut: $] ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:02'!
writeObject: object streamMap: block
	self writeObject: object do: [ | mapWriter |
		mapWriter := STONMapWriter on: self.
		writeStream nextPut: ${.
		self indentedDo: [
			self newlineIndent.
			block value: mapWriter ].
		self newlineIndent.
		writeStream nextPut: $} ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/24/2012 10:02'!
writeObject: object streamShortList: block
	self writeObject: object do: [ | listWriter |
		listWriter := STONShortListWriter on: self.
		writeStream nextPut: $[.
		self indentedDo: [
			self prettyPrintSpace.
			block value: listWriter ].
		self prettyPrintSpace.
		writeStream nextPut: $] ]! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/14/2012 17:29'!
writeReference: index
	writeStream
		nextPut: $@;
		print: index! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 2/29/2012 21:49'!
writeString: string
	self encodeString: string! !

!STONWriter methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 4/27/2012 13:52'!
writeSymbol: symbol
	jsonMode
		ifTrue: [
			self writeString: symbol ]
		ifFalse: [
			writeStream nextPut: $#.
			(self isSimpleSymbol: symbol)
				ifTrue: [
					writeStream nextPutAll: symbol ]
				ifFalse: [
					self encodeString: symbol ] ]! !

!DateAndTime class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 5/28/2015 14:14'!
fromSton: stonReader
	^ self readFrom: stonReader parseListSingleton readStream! !

!DateAndTime methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:25'!
stonContainSubObjects 
	^ false! !

!DateAndTime methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 20:03'!
stonOn: stonWriter
	"Use an ISO representation with all details"
	
	stonWriter writeObject: self listSingleton: 
		(String streamContents: [ :stream |
			self printOn: stream withLeadingSpace: false ])! !

!String methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:27'!
stonContainSubObjects 
	^ false! !

!String methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/14/2012 16:22'!
stonOn: stonWriter
	stonWriter writeString: self! !

!OrderedDictionary class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 4/10/2015 15:29'!
fromSton: stonReader
	"My instances are stored as maps."
	
	| dictionary |
	dictionary := self new.
	stonReader parseMapDo: [ :key :value |
		dictionary at: key put: value ].
	^ dictionary! !

!OrderedDictionary methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 4/10/2015 15:28'!
stonOn: stonWriter
	"I store my instances as maps"
	
	stonWriter 
		writeObject: self 
		do: [ stonWriter encodeMap: self ]! !

!Symbol methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/14/2012 16:22'!
stonOn: stonWriter
	stonWriter writeSymbol: self! !

!Character class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 15:17'!
fromSton: stonReader
	^ stonReader parseListSingleton first! !

!Character methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 20:02'!
stonOn: stonWriter
	stonWriter writeObject: self listSingleton: self asString! !

!Metaclass class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 12/24/2015 20:59'!
fromSton: stonReader
	| theClassName theClass theMetaclass |
	theClassName := stonReader parseListSingleton.
	theClass := self environment at: theClassName.
	theMetaclass := theClass class.
	^ theMetaclass! !

!ByteArray class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 15:16'!
fromSton: stonReader
	^ self readHexFrom: stonReader parseListSingleton! !

!ByteArray methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:25'!
stonContainSubObjects 
	^ false! !

!ByteArray methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 20:02'!
stonOn: stonWriter
	"Use a hex representation"
	
	stonWriter writeObject: self listSingleton: self hex! !

!Interval class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/24/2016 13:57'!
fromSton: stonReader
	^ self new
		fromSton: stonReader;
		yourself! !

!Interval methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/24/2016 13:56'!
fromSton: stonReader
	stonReader parseNamedInstVarsFor: self! !

!Interval methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/24/2016 13:56'!
stonOn: stonWriter
	stonWriter writeObject: self! !

!Integer methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/14/2012 15:47'!
stonOn: stonWriter
	stonWriter writeInteger: self! !

!TClassDescription methodsFor: '*ston-core' stamp: 'PavelKrivanek 4/20/2016 12:25'!
stonContainSubObjects
	^ false! !

!Date class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 15:17'!
fromSton: stonReader
	^ self readFrom: stonReader parseListSingleton readStream! !

!Date methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:26'!
stonContainSubObjects 
	^ false! !

!Date methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 20:03'!
stonOn: stonWriter
	"Use an ISO style YYYYMMDD representation"
	
	stonWriter writeObject: self listSingleton: self yyyymmdd! !

!Association methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 8/15/2015 17:12'!
stonOn: stonWriter
	self class == STON associationClass
		ifTrue: [ stonWriter writeAssociation: self ]
		ifFalse: [ super stonOn: stonWriter ]
	! !

!Boolean methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:24'!
stonContainSubObjects 
	^ false! !

!Boolean methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/14/2012 15:53'!
stonOn: stonWriter
	stonWriter writeBoolean: self! !

!Text class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 3/23/2014 18:48'!
fromSton: stonReader
	^ self new
		fromSton: stonReader;
		yourself! !

Error subclass: #STONReaderError
	instanceVariableNames: 'streamPosition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONReaderError commentStamp: '<historical>' prior: 0!
STONReaderError is the error/exception signalled by STONReader when illegal/incorrect input is seen. 
!

!STONReaderError class methodsFor: 'instance creation' stamp: 'DamienCassou 12/9/2013 10:04'!
signal: aString streamPosition: streamPosition 
	^ self new
		streamPosition: streamPosition;
		signal: aString;
		yourself! !

!STONReaderError methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/13/2013 16:12'!
messageText
	^ streamPosition 
		ifNil: [ 
			super messageText ] 
		ifNotNil: [ :pos | 
			'At character {1}: {2}' format: 
				(Array with: streamPosition with: super messageText) ]! !

!STONReaderError methodsFor: 'accessing' stamp: 'DamienCassou 12/9/2013 10:00'!
streamPosition
	^ streamPosition! !

!STONReaderError methodsFor: 'accessing' stamp: 'DamienCassou 12/9/2013 10:00'!
streamPosition: aNumber
	streamPosition := aNumber! !

Error subclass: #STONWriterError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'STON-Core'!

!STONWriterError commentStamp: 'SvenVanCaekenberghe 4/26/2012 17:17' prior: 0!
STONWriterError is the error/exception signalled by STONWriter when illegal/incorrect input is seen. !

!Point methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 14:30'!
fromSton: stonReader
	stonReader parseListDo: [ :each :index |
		index = 1 ifTrue: [ x := each ].
		index = 2 ifTrue: [ y := each ] ]! !

!Point methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/24/2012 10:02'!
stonOn: stonWriter
	stonWriter writeObject: self streamShortList: [ :array |
		array add: x; add: y ]! !

!RunArray class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 3/23/2014 18:51'!
fromSton: stonReader
	^ self new
		fromSton: stonReader;
		yourself! !

!SmallDictionary class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 9/17/2013 19:50'!
fromSton: stonReader
	"My instances are stored as maps."
	
	| dictionary |
	dictionary := self new.
	stonReader parseMapDo: [ :key :value |
		dictionary at: key put: value ].
	^ dictionary! !

!SmallDictionary methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 9/17/2013 20:13'!
stonOn: stonWriter
	"I store my instances as maps"
	
	stonWriter 
		writeObject: self 
		do: [ stonWriter encodeMap: self ]! !

!TClass methodsFor: '*ston-core' stamp: 'PavelKrivanek 4/20/2016 12:31'!
stonOn: stonWriter
	stonWriter 
		writeObject: self 
		listSingleton: self name asSymbol! !

!SequenceableCollection class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 15:07'!
fromSton: stonReader
	^ self streamContents: [ :stream |
		stonReader parseListDo: [ :each |
			stream nextPut: each ] ]! !

!SequenceableCollection methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/8/2012 19:22'!
stonOn: stonWriter
	self class == STON listClass
		ifTrue: [ stonWriter writeList: self ]
		ifFalse: [ super stonOn: stonWriter ]! !

!Number methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:26'!
stonContainSubObjects 
	^ false! !

!Number methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/14/2012 17:26'!
stonOn: stonWriter
	stonWriter writeFloat: self asFloat! !

!Time class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 15:17'!
fromSton: stonReader
	^ self readFrom: stonReader parseListSingleton readStream! !

!Time methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:26'!
stonContainSubObjects 
	^ false! !

!Time methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 20:03'!
stonOn: stonWriter
	"Use an ISO style HH:MM:SS representation"
	 
	stonWriter writeObject: self listSingleton: 
		(String streamContents: [ :stream |
			self print24: true on: stream ])! !

!UndefinedObject methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 11/29/2012 16:27'!
stonContainSubObjects 
	^ false! !

!UndefinedObject methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 1/14/2012 15:54'!
stonOn: stonWriter
	stonWriter writeNull! !

!Class class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 12/24/2015 20:58'!
fromSton: stonReader
	| theClassName theClass |
	theClassName := stonReader parseListSingleton.
	theClass := self environment at: theClassName.
	^ theClass! !

!TApplyingOnClassSide methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/22/2016 12:33'!
stonName
	^ #Class! !

!TApplyingOnClassSide methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/22/2016 12:33'!
stonOn: stonWriter
	stonWriter
		writeObject: self 
		listSingleton: self theNonMetaClass name asSymbol! !

!Collection class methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 2/12/2012 13:54'!
fromSton: stonReader
	| collection |
	collection := self new.
	stonReader parseListDo: [ :each |
		collection add: each ].
	^ collection! !

!Collection methodsFor: '*ston-core' stamp: 'SvenVanCaekenberghe 4/29/2012 12:23:33'!
stonOn: stonWriter
	stonWriter writeObject: self do: [
		stonWriter encodeList: self ]
! !

STONWriter initialize!
